#!/usr/bin/python
# -*- encoding: utf-8 -*-

import RPi.GPIO as GPIO
import time
import smbus
import random




bus = smbus.SMBus(1)   # novejsi varianta RasPi (512MB)
addr = 0x40            # I2C adresa obvodu PCA9685 (kdyz jsou vsechny adresovaci vstupy v "0", je adresa 0x40)

GPIO.setwarnings(False)
GPIO.setmode(GPIO.BCM)


gen=25000000                         # frekvence vnitrniho generatoru obvodu PCA9685 (25MHz)
frekvence = 50                       # pozadovana frekvence vystupu (50Hz)
n = int((gen /frekvence) / 4096)+1   # prepocet na pozadovanou konstantu prescaleru

bus.write_byte_data(addr,0x00,0x10)  # STOP - zastaveni generatoru
bus.write_byte_data(addr,0x00,0x90)  # restart obvodu
bus.write_byte_data(addr,0xFE,n)     # nastaveni frekvence prescaleru
bus.write_byte_data(addr,0x01,0x04)  # Logika vystupu: Vystup je aktivni pri "1" ("H" = Vdd ; "L" = GND)

while True:
 
  # v prikladu se pro nataceni serv generuji nahodna cisla v rozsahu 0.98ms az 1.95ms (cisla 200 az 400)
  sirka_h = random.randint(200, 400)   # sirka pulzu pro horizontalni servo (cislo 103 az 512 odpovida casu 0.5ms az 2.5ms)
  sirka_v = random.randint(200, 400)   # sirka pulzu pro vertikalni servo (cislo 103 az 512 odpovida casu 0.5ms az 2.5ms)
 
  
  
  # nastaveni sirky impulzu pro horizontalni servo
  start_adr_kan_h = 0x0A                               # horizontalni servo je na vyvodu LED1 obvodu PCA9685 (registry 10 az 13)
  bus.write_byte_data(addr, start_adr_kan_h,     0x00) # H. kanal - prechod z "0" na "1" v case "T + nula"  8xLSB
  bus.write_byte_data(addr, start_adr_kan_h + 1, 0x00) #                                                    4xMSB
 
  dolnicast_h = sirka_h % 256                          # prepocet sirky horizontalniho impulzu na horni a dolni bajt
  hornicast_h = int(sirka_h/256)
  bus.write_byte_data(addr, start_adr_kan_h + 2, dolnicast_h) # H. kanal - prechod z "1" na "0" 8xLSB
  bus.write_byte_data(addr, start_adr_kan_h + 3, hornicast_h) #                                 4xMSB
  
  
  # nastaveni sirky impulzu pro vertikalni servo
  start_adr_kan_v = 0x06                               # vertikalni servo je na vyvodu LED0 obvodu PCA9685 (registry 6 az 9)
  bus.write_byte_data(addr, start_adr_kan_v    , 0x00) # V. kanal - prechod z "0" na "1" v case "T + nula" 8xLSB
  bus.write_byte_data(addr, start_adr_kan_v + 1, 0x00) #                                                   4xMSB
  
  dolnicast_v = sirka_v % 256                          # prepocet sirky vertikalniho impulzu na horni a dolni bajt
  hornicast_v = int(sirka_v/256)
  bus.write_byte_data(addr, start_adr_kan_v + 2, dolnicast_v) # V. kanal - prechod z "1" na "0" 8xLSB
  bus.write_byte_data(addr, start_adr_kan_v + 3, hornicast_v) #                                 4xMSB
  

  # spusteni pohonu na jednu sekundu    
  bus.write_byte_data(addr,0x00,0x00)  # spust PWM generatory
  time.sleep(1)                        # 1 sekunda je dostacujici pro najeti z jedne strany na druhou
  bus.write_byte_data(addr,0x00,0x10)  # zastav PWM generatory
  


  # na zaver dojde k vystrelu davky laserem
  laser_pin = 7                          # GPIO7 (= hw pin 26)
  GPIO.setup(laser_pin, GPIO.OUT)        # GPIO pro ovladani laseru prepne na vystup
  signal3 = GPIO.PWM(laser_pin , 5)      # frekvence (kadence) vystrelu 5Hz 
  signal3.start(20)                      # palba (20% strida)
  time.sleep(2)
  signal3.stop()                         # po 2 sekundach se palba zastavi
  GPIO.output(laser_pin, False)          # pro jistotu se laser vypne




